import re
import time
import sys
import socket
from flask import request, g
from flask import _app_ctx_stack as stack
from statsd import StatsClient


def add_tags(metric, **tags):
    if not metric:
        return metric
    tag_str = ','.join([('%s=%s' % (k, v)) for k, v in tags.items()])
    return '%s,%s' % (metric, tag_str)

def _get_context_tags():
    try:
        return g.statsd_tags
    except AttributeError:
        return {}


class FlaskStatsdTagged(object):

    def __init__(self, app=None, host='localhost', port=8125, extra_tags={}):
        self.app = app
        self.hostname = socket.gethostname()
        self.statsd_host = host
        self.statsd_port = port
        self.extra_tags = extra_tags
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        app.before_request(self.before_request)
        app.after_request(self.after_request)
        self.connection = self.connect()

    def connect(self):
        return StatsClient(host=self.statsd_host,
                           port=self.statsd_port,
                           maxudpsize=1024)

    def before_request(self):
        ctx = stack.top
        ctx.request_begin_at = time.time()

    def after_request(self, resp):
        ctx = stack.top
        period = (time.time() - ctx.request_begin_at) * 1000
        tags = dict(self.extra_tags)
        tags.update({"path":request.path or "notfound", "server": self.hostname, "status_code": resp.status_code})
        tags.update(_get_context_tags())
        with self.pipeline() as pipe:
            metric = add_tags("flaskrequest", **tags)
            pipe.incr(metric)
            pipe.timing(metric, period)

        return resp

    def pipeline(self):
        return self.connection.pipeline()
